@@ -3,6 +3,8 @@
 namespace PHPStan\PhpDocParser\Ast\Type;
 
 use PHPStan\PhpDocParser\Ast\NodeAttributes;
+use function array_map;
+use function implode;
 
 class UnionTypeNode implements TypeNode
 {
@@ -10,8 +12,11 @@ class UnionTypeNode implements TypeNode
 	use NodeAttributes;
 
 	/** @var TypeNode[] */
-	public $types;
+	public array $types;
 
+	/**
+	 * @param TypeNode[] $types
+	 */
 	public function __construct(array $types)
 	{
 		$this->types = $types;
@@ -20,7 +25,13 @@ class UnionTypeNode implements TypeNode
 
 	public function __toString(): string
 	{
-		return '(' . implode(' | ', $this->types) . ')';
+		return '(' . implode(' | ', array_map(static function (TypeNode $type): string {
+			if ($type instanceof NullableTypeNode) {
+				return '(' . $type . ')';
+			}
+
+				return (string) $type;
+		}, $this->types)) . ')';
 	}
 
 }
