@@ -8,20 +8,22 @@ use PHPStan\PhpDocParser\Parser\ConstExprParser;
 use PHPStan\PhpDocParser\Parser\PhpDocParser;
 use PHPStan\PhpDocParser\Parser\TokenIterator;
 use PHPStan\PhpDocParser\Parser\TypeParser;
+use PHPStan\PhpDocParser\ParserConfig;
 use PHPUnit\Framework\TestCase;
 
 final class AttributesTest extends TestCase
 {
 
-	/** @var PhpDocNode */
-	private $phpDocNode;
+	private PhpDocNode $phpDocNode;
 
 	protected function setUp(): void
 	{
 		parent::setUp();
-		$lexer = new Lexer();
-		$constExprParser = new ConstExprParser();
-		$phpDocParser = new PhpDocParser(new TypeParser($constExprParser), $constExprParser);
+
+		$config = new ParserConfig([]);
+		$lexer = new Lexer($config);
+		$constExprParser = new ConstExprParser($config);
+		$phpDocParser = new PhpDocParser($config, new TypeParser($config, $constExprParser), $constExprParser);
 
 		$input = '/** @var string */';
 		$tokens = new TokenIterator($lexer->tokenize($input));
